//--------------------------------------------------------------------------
#include <stddef.h>
#include <math.h>
#include <mem.h>
#pragma hdrstop

#include <LibRand.h>
#include <LibBiophysics.h>

#include "Cell.h"
//---------------------------------------------------------------------------

#pragma package(smart_init)

CCell::CCell(void):CSimpleCell()
{

  Filament=(CFilament*)malloc(nfmax*sizeof(CFilament));
  memset(Filament, 0, nfmax*sizeof(CFilament));

}

CCell::~CCell(void)
{
  if (nfmax>0) free(Filament);
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CCell::AddFilament(int nactin)
{
  int i;
  int *TMP;

  //check, if nf is in range of F, otherwise - resize F
  if ((nf+1)>nfmax){
    nfmax+=size_step;
    Filament=(CFilament*)realloc(Filament,sizeof(CFilament)*nfmax);
    if (Filament==NULL) return 0;
  }
  Filament[nf].natf=nactin;
  Filament[nf].nadf=0;
  Filament[nf].b=iATF;
  Filament[nf].p=iATF;

//  Filament[nf].nfluo=0;
  nf++;

  return 1;
}

//---------------------------------------------------------------------------
int  CCell::DelFilament(int ifil)
{
  if (ifil>=nf) return -1;
  nf--;
  Filament[ifil].natf=Filament[nf].natf;
  Filament[ifil].nadf=Filament[nf].nadf;
  Filament[ifil].b=Filament[nf].b;
  Filament[ifil].p=Filament[nf].p;
  return 1;
}

//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int CCell::SimDissociate(int ifil)
{
  if (ifil>=nf)   return -1;

  dis_count++;
  dis_actin+=Filament[ifil].natf+Filament[ifil].nadf;

  int nfib;

  // actins
  N[iATG]+=Filament[ifil].natf;
  N[iADG]+=Filament[ifil].nadf;
  N[iATF]-=Filament[ifil].natf;
  N[iADF]-=Filament[ifil].nadf;

  // barbed ends
  switch (Filament[ifil].b){
   case iATF: N[iFTB]--; break;
   case iADF: N[iFDB]--; break;
   case iCBF: N[iCBF]--; N[iCBG]++; break;
   case iFOF: N[iFOF]--; N[iFOG]++; break;
  }

  // pointed ends
  switch (Filament[ifil].p){
   case iATF: N[iFTP]--; break;
   case iADF: N[iFDP]--; break;
   case iCPF: N[iCPF]--; N[iCPG]++; break;
  }

  //nfluo-=Filament[ifil].nfluo;
  return DelFilament(ifil);
}
//---------------------------------------------------------------------------
// NUCLEATIONS
//---------------------------------------------------------------------------
int CCell::SimSNUC(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;
  N[iATG]-=3;
  N[iATF]+=3;
  N[iFTB]++;
  N[iFTP]++;

  //Filament[nf-1].nfluo=3;
  //nfluo+=3;

  NR[iSNUC]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimFNUC(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;

  // formin change ATF->ADF
  Filament[nf-1].nadf=3;
  Filament[nf-1].natf=0;
  Filament[nf-1].b=iFOF;
  Filament[nf-1].p=iADF;

  N[iATG]-=3;
  N[iADF]+=3;
  N[iFOF]++;
  N[iFOG]--;
  N[iFDP]++;


  NR[iSNUC]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimCBNU(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;

  Filament[nf-1].b=iCBF;

  N[iATG]-=3;
  N[iATF]+=3;
  N[iCBF]++;
  N[iFTP]++;

  NR[iSNUC]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimCPNU(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;

  Filament[nf-1].p=iCPF;

  N[iATG]-=3;
  N[iATF]+=3;
  N[iFTB]++;
  N[iCPF]++;

  NR[iSNUC]++;
  return 1;
}
//---------------------------------------------------------------------------
// ASSOCIATIONS
//---------------------------------------------------------------------------
int CCell::SimASTB(void)
{
  int tmp,if1;
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))) return -3;

  Filament[ifil].natf++; // add ATP-actin to filament
  if (Filament[ifil].b==iADF) { // check whether the old b-end was the same
    N[iFDB]--;                  // change b-end if not
    N[iFTB]++;
    Filament[ifil].b=iATF;
  }

  N[iATG]--;
  N[iATF]++;


  NR[iASTB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimASDB(void)
{
  int tmp,if1;
  int i,j, ifil;
  if (N[iADG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {            // nf-1 ???
    if ((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))) return -3;

  Filament[ifil].nadf++;
  N[iADG]--;
  N[iADF]++;

  if (Filament[ifil].b==iATF) { // check whether the old b-end was the same
    N[iFTB]--;                  // change b-end if not
    N[iFDB]++;
    Filament[ifil].b=iADF;
  }

  NR[iASDB]++;

  return 1;
}

//---------------------------------------------------------------------------
int CCell::SimASTP(void)
{
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFDP]+N[iFTP]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTP]+N[iFDP]);
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if ((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))) return -3;

  Filament[ifil].natf++;
  N[iATG]--;
  N[iATF]++;

  if (Filament[ifil].p==iADF) { // check whether the old b-end was the same
    N[iFDP]--;                  // change b-end if not
    N[iFTP]++;
    Filament[ifil].p=iATF;
  }


  NR[iASTP]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimASDP(void)
{
  int i,j,ifil;
  if (N[iADG]<1) return -1;
  if (N[iFDP]+N[iFTP]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTP]+N[iFDP]);
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {           // nf-1 ???
    if ((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))) return -3;

  Filament[ifil].nadf++;
  if (Filament[ifil].p==iATF) { // check whether the old b-end was the same
    N[iFTP]--;                  // change b-end if not
    N[iFDP]++;
    Filament[ifil].p=iADF;
  }

  N[iADG]--;
  N[iADF]++;

  NR[iASDP]++;

  return 1;
}
//---------------------------------------------------------------------------
// DISSOCIATIONS
//---------------------------------------------------------------------------
int CCell::SimDITB(void)
{
  int i,j, ifil;
  if (N[iATF]<1) return -1;
  if (N[iFTB]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFTB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil].b==iATF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].b==iATF)) return -3;

  Filament[ifil].natf--;
  N[iATG]++;
  N[iATF]--;

  if ((Filament[ifil].natf==0)||   // should we change b-end?
      ((Filament[ifil].natf==1)&&(Filament[ifil].p==iATF))){
    N[iFDB]++;                 // change b-end if yes
    N[iFTB]--;
    Filament[ifil].b=iADF;
  }

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil].nadf+Filament[ifil].natf<3)&&KillShortFilament)
    SimDissociate(ifil);

  NR[iDITB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDIDB(void)
{
  int i,j, ifil;
  if (N[iADF]<1) return -1;
  if (N[iFDB]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {                // nf-1 ???
    if (Filament[ifil].b==iADF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].b==iADF)) return -3;

  Filament[ifil].nadf--;
  N[iADG]++;
  N[iADF]--;
  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil].nadf+Filament[ifil].natf<3)&&KillShortFilament)
    SimDissociate(ifil);
  else {
    if ((Filament[ifil].natf>0)&&
        (!((Filament[ifil].natf==1)&&(Filament[ifil].p==iATF)))){
      N[iFDB]--;                 // change b-end if yes
      N[iFTB]++;
      Filament[ifil].b=iATF;
    }
  } //else

  NR[iDIDB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDITP(void)
{
  int i,j, ifil;

  if (N[iATF]<1) return -1;
  if (N[iFTP]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFTP]); // get random barbed end        // was (0,N[iFTP]-1)
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil].p==iATF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].p==iATF)) return -3;

  Filament[ifil].natf--;
  N[iATG]++;
  N[iATF]--;


  if ((Filament[ifil].natf==0)||   // should we change p-end?
      ((Filament[ifil].natf==1)&&(Filament[ifil].b==iATF))){
    N[iFDP]++;                 // change b-end if yes
    N[iFTP]--;
    Filament[ifil].p=iADF;
  }

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil].nadf+Filament[ifil].natf<3)&&KillShortFilament)
    SimDissociate(ifil);

  NR[iDITP]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDIDP(void)
{
  int i,j, ifil;
  if (N[iADF]<1) return -1;
  if (N[iFDP]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFDP]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {             // nf-1 ???
    if (Filament[ifil].p==iADF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].p==iADF)) return -3;

  Filament[ifil].nadf--;
  N[iADG]++;
  N[iADF]--;

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil].nadf+Filament[ifil].natf<3)&&KillShortFilament)
    SimDissociate(ifil);
  else {

    if ((Filament[ifil].natf>0)&&
        (!((Filament[ifil].natf==1)&&(Filament[ifil].b==iATF)))){
      N[iFDP]--;                 // change b-end if yes
      N[iFTP]++;
      Filament[ifil].p=iATF;
    }
  } //else

  NR[iDIDB]++;

  return 1;
}
//---------------------------------------------------------------------------
// AGING AND ATP-RECHARGE
//---------------------------------------------------------------------------
int CCell::SimTTOD(void)
{
  int i,j,ifil;
  if (N[iATF]<1) return -1;
  // select the aged actin
  i=RandInt(1,N[iATF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    j+=Filament[ifil].natf;
    if (j>=i) break;
  }

  // track changes in the b-ends due to randomness
  if ((Filament[ifil].b==iATF)&&(rand()<1.0/Filament[ifil].natf)){
    Filament[ifil].b=iADF;
    N[iFTB]--;
    N[iFDB]++;
  }
  // track changes in the p-ends due to randomness
  else if ((Filament[ifil].p==iATF)&&(rand()<1.0/Filament[ifil].natf)){
    Filament[ifil].p=iADF;
    N[iFTP]--;
    N[iFDP]++;
  }

  Filament[ifil].natf--;
  Filament[ifil].nadf++;
  N[iATF]--;
  N[iADF]++;

  // track changes in the ends due to lack of ATF
  if (Filament[ifil].natf==0){   // no ATF any more
    if (Filament[ifil].b==iATF) {
      Filament[ifil].b=iADF;
      N[iFTB]--;
      N[iFDB]++;
    } // if (Filament[ifil].b==iATF)
    if (Filament[ifil].p==iATF) {
      Filament[ifil].p=iADF;
      N[iFTP]--;
      N[iFDP]++;
    } // if (Filament[ifil].p==iATF)
  } // if (Filament[ifil].natf==0)

  if ((Filament[ifil].natf==1)&&(Filament[ifil].b==iATF)&&(Filament[ifil].p==iATF)){   // single ATF is busy
    Filament[ifil].p=iADF;
    N[iFTP]--;
    N[iFDP]++;
  }

  NR[iTTOD]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDTOT(void)
{
  if (N[iADG]<1) return -1;
  N[iADG]--;
  N[iATG]++;

  NR[iDTOT]++;
  return 1;
}
//---------------------------------------------------------------------------
// CAPPING
//---------------------------------------------------------------------------
int CCell::SimASCB(void)
{
  int i,j, ifil;
  if (N[iCBG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))) return -3;

  N[iCBG]--;
  N[iCBF]++;

  // change the end
  if (Filament[ifil].b==iATF) N[iFTB]--;
  if (Filament[ifil].b==iADF) N[iFDB]--;
  Filament[ifil].b=iCBF;

  NR[iASCB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDICB(void)
{
  int i,j, ifil;
  if (N[iCBF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iCBF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil].b==iCBF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].b==iCBF)) return -2;

  N[iCBG]++;
  N[iCBF]--;

  // change the end
  if ((Filament[ifil].natf>0)&&
       (!((Filament[ifil].natf==1)&&(Filament[ifil].p==iATF)))){
    // set ATF to barbed
    N[iFTB]++;
    Filament[ifil].b=iATF;
  } else {
    // set ADF to barbed
    N[iFDB]++;
    Filament[ifil].b=iADF;
  }

  NR[iDICB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimASCP(void)
{
  int i,j, ifil;
  if (N[iCPG]<1) return -1;
  if (N[iFTP]+N[iFDP]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTP]+N[iFDP]); // get random pointed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].p==iATF)||(Filament[ifil].p==iADF))) return -3;

  N[iCPG]--;
  N[iCPF]++;

  // change the end
  if (Filament[ifil].p==iATF) N[iFTP]--;
  if (Filament[ifil].p==iADF) N[iFDP]--;
  Filament[ifil].p=iCPF;

  NR[iASCP]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDICP(void)
{
  int i,j, ifil;
  if (N[iCPF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iCPF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil].p==iCPF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].p==iCPF)) return -2;

  N[iCPG]++;
  N[iCPF]--;

  // change the end
  if ((Filament[ifil].nadf>0)&&
       (!((Filament[ifil].nadf==1)&&(Filament[ifil].b==iADF)))){
    // set ADF to pointed
    N[iFDP]++;
    Filament[ifil].p=iADF;
  } else {
    // set ATF to pointed
    N[iFTP]++;
    Filament[ifil].p=iATF;
  }

  NR[iDICP]++;
  return 1;
}
//---------------------------------------------------------------------------
// FORMIN OPERATION
//---------------------------------------------------------------------------
int CCell::SimASFB(void)
{
  int i,j, ifil;
  if (N[iFOG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil].b==iATF)||(Filament[ifil].b==iADF))) return -3;

  N[iFOG]--;
  N[iFOF]++;

  // change the end
  if (Filament[ifil].b==iATF) N[iFTB]--;
  if (Filament[ifil].b==iADF) N[iFDB]--;
  Filament[ifil].b=iFOF;

  NR[iASFB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimDIFB(void)
{
  int i,j, ifil;
  if (N[iFOF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iFOF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil].b==iFOF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].b==iFOF)) return -2;

  N[iFOG]++;
  N[iFOF]--;

  // aging of the filament
  Filament[ifil].nadf+=Filament[ifil].natf;
  N[iADF]+=Filament[ifil].natf;
  N[iATF]-=Filament[ifil].natf;
  Filament[ifil].natf=0;
  Filament[ifil].b=iADF;
  N[iFDB]++;
  if (Filament[ifil].p==iATF) {
    Filament[ifil].p=iADF;
    N[iFTP]--;
    N[iFDP]++;
  }

  NR[iDIFB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCell::SimFASB(void)
{
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFOF]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFOF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil].b==iFOF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil].b==iFOF)) return -3;

  Filament[ifil].nadf++; // add ADP-actin to filament (formin discharge)

  N[iATG]--;
  N[iADF]++;

  NR[iFASB]++;
  return 1;
}
//----------------------------------------------------------------------------
int CCell::GetLength3(void)
{
  int ifil,n3=0;
  for (ifil=0;ifil<nf;ifil++)
   if (Filament[ifil].nadf+Filament[ifil].natf == 3)
     n3++;
  return n3;
}
//----------------------------------------------------------------------------
int CCell::RetFilLen(int nm){
	return Filament[nm].nadf+Filament[nm].natf;
}
//----------------------------------------------------------------------------
int CCell::GetMinLength(void)
{
  int minl=1e6;
  for (int ifil=0; ifil<nf; ifil++){
    if (Filament[ifil].nadf+Filament[ifil].natf < minl)
      minl=Filament[ifil].nadf+Filament[ifil].natf;
  }
  return minl;
}
//----------------------------------------------------------------------------
int CCell::GetMaxLength(void)
{
  int maxl=0;
  for (int ifil=0; ifil<nf; ifil++){
    if (Filament[ifil].nadf+Filament[ifil].natf > maxl)
      maxl=Filament[ifil].nadf+Filament[ifil].natf;
  }
  return maxl;
}

//----------------------------------------------------------------------------
void CCell::GetLengthDistribution(int *B, int *F, int _vmin, int _vmax, int nbins)
{
  int vmin, vmax;
  int s;
  int i,ifil;

  if (_vmin<0) vmin=GetMinLength();
  else vmin=_vmin;
  if (_vmax<0) vmax=GetMaxLength();
  else vmax=_vmax;

  for (i=0;i<nbins;i++) {
    B[i]=vmin+int((double)(vmax-vmin)*(i+1.0)/nbins);//!!!round
    F[i]=0;
  }

  for (ifil=0; ifil<nf; ifil++){
    for (i=0;i<nbins;i++)
      if (Filament[ifil].nadf+Filament[ifil].natf<=B[i]){
        F[i]=F[i]+1;
        break;
      }
  }

}

//---------------------------------------------------------------------------
