//clesses: CFilament, CCell
//---------------------------------------------------------------------------
#ifndef CellH
#define CellH
//---------------------------------------------------------------------------
#include "SimpleCell.h"


struct CFilament{
  int natf;   //number of ATP-actines
  int nadf;   //number of ADP-actines
  char b;  // barbed
  char p;  // pointed
};


class CCell : public CSimpleCell{
 public:

  CFilament *Filament;

  CCell(void);
  virtual ~CCell(void);

  // calculate mean filament length
  virtual int GetMinLength(void);
  virtual int GetMaxLength(void);
  virtual void GetLengthDistribution(int *B, int *F, int vmin, int vmax, int nbins);

  // Return number of filaments with l=3
  virtual int GetLength3(void);

  //Return length of filament with index nm
  virtual int RetFilLen(int nm);

  // add filament to the system
  virtual int AddFilament(int nactin);
  // delete filament from the system
  virtual int DelFilament(int ifil);

  // Simulate dissociation of the filament #ifil
  virtual int SimDissociate(int ifil);

  // Simulate stochastic filament formation
  virtual int SimSNUC(void);
  // Simulate formin filament formation
  virtual int SimFNUC(void);
  // Simulate barbed capping protein nuctleation
  virtual int SimCBNU(void);
  // Simulate pointed capping protein nuctleation
  virtual int SimCPNU(void);


  // Simulate ATP-actin association at barbed end
  virtual int SimASTB(void);
  // Simulate ADP-actin association at barbed end
  virtual int SimASDB(void);
  // Simulate ATP-actin association at pointed end
  virtual int SimASTP(void);
  // Simulate ADP-actin association at pointed end
  virtual int SimASDP(void);


  // Simulate dissociation at ATP-barbed end
  virtual int SimDITB(void);
  // Simulate dissociation at ADP-barbed end
  virtual int SimDIDB(void);
  // Simulate dissociation at ATP-pointed end
  virtual int SimDITP(void);
  // Simulate dissociation at ADP-pointed end
  virtual int SimDIDP(void);

  // Simulate aging of F-actins
  virtual int SimTTOD(void);
  // Simulate recharge of G-actins
  virtual int SimDTOT(void);

  // Capping of barbed end
  virtual int SimASCB(void);
  // Uncapping of barbed end
  virtual int SimDICB(void);
  // Capping of pointed end
  virtual int SimASCP(void);
  // Uncapping of pointed end
  virtual int SimDICP(void);

  // Association of formin to barbed end
  virtual int SimASFB(void);
  // Dissociation of formin from barbed end
  virtual int SimDIFB(void);
  // Formin-initiated barbed-end association
  virtual int SimFASB(void);

};

#endif




