//--------------------------------------------------------------------------
#include <stddef.h>
#include <math.h>
#include <mem.h>
#pragma hdrstop

#include <LibRand.h>
#include <LibBiophysics.h>

#include "CellStruct.h"
//---------------------------------------------------------------------------

#pragma package(smart_init)

char Reform_GF_FG (char mol) {
  switch (mol) {
   case iATG: return iATF;
   case iADG: return iADF;
   case iCBG: return iCBF;
   case iCPG: return iCPF;
   case iFOG: return iFOF;
   case iATF: return iATG;
   case iADF: return iADG;
   case iCBF: return iCBG;
   case iCPF: return iCPG;
   case iFOF: return iFOG;
  }
  return -1;
}

char Trans_A_FB (char mol)
{
  switch (mol) {
   case iATF: return iFTB;
   case iADF: return iFDB;
  }
  return -1;
}

char Trans_A_FP (char mol)
{
  switch (mol) {
   case iATF: return iFTP;
   case iADF: return iFDP;
  }
  return -1;
}

//---------------------------------------

CFilamentStruct::CFilamentStruct(void)
{
  Barbed = Pointed = NULL;
  n = 0;
  natf=nadf=0;
  b=p=-1;
}

CFilamentStruct::~CFilamentStruct(void)
{
  while (n>0) RemBarbed();
}

void CFilamentStruct::AddBarbed(char mol)
{
  char molnw=mol;
  TElement *noval;
  noval = new TElement;

  noval->mol=molnw;

  if (b==iFOF) {
    noval->mol=iFOF;
    molnw=iADF;
    Barbed->mol=molnw;
  }

  noval->next=NULL;
  noval->prev=Barbed;
  if (Barbed!=NULL)
    Barbed->next=noval;
  Barbed=noval;
  if (Pointed==NULL)
    Pointed=noval;

  n++;

  if (molnw==iATF) natf++;
  if (molnw==iADF) nadf++;

  b=Barbed->mol;
  p=Pointed->mol;
}

void CFilamentStruct::AddPointed(char mol)
{
  TElement *noval;
  noval = new TElement;
  noval->mol=mol;
  noval->prev=NULL;
  noval->next=Pointed;
  if (Pointed!=NULL)
    Pointed->prev=noval;
  Pointed=noval;
  if (Barbed==NULL)
    Barbed=noval;

  n++;

  if (mol==iATF) natf++;
  if (mol==iADF) nadf++;

  b=Barbed->mol;
  p=Pointed->mol;
}

char CFilamentStruct::RemBarbed(void)
{
  if (n<1) return -1;
  char mol=Barbed->mol;
  TElement *temp;

  temp=Barbed;
  Barbed=Barbed->prev;
  delete temp;
  if (Barbed!=NULL)
    Barbed->next=NULL;
  else Pointed=NULL;
  n--;

  if (mol==iATF) natf--;
  if (mol==iADF) nadf--;

  if (Barbed!=NULL) b=Barbed->mol;  else b=-1;
  if (Pointed!=NULL) p=Pointed->mol; else p=-1;

  return mol;
}

char CFilamentStruct::RemPointed(void)
{
  if (n<1) return -1;
  TElement *temp;
  char mol=Pointed->mol;

  temp=Pointed;
  Pointed=Pointed->next;
  delete temp;
  if (Pointed!=NULL)
    Pointed->prev=NULL;
  else Barbed=NULL;
  n--;

  if (mol==iATF) natf--;
  if (mol==iADF) nadf--;
  if (Barbed!=NULL) b=Barbed->mol; else b=-1;
  if (Pointed!=NULL) p=Pointed->mol; else p=-1;

  return mol;
}

int CFilamentStruct::GoToElement(int cnt)
{
  if (cnt>=n) return -1;
  Element=Pointed;
  for (int i=0;i<cnt;i++)
    Element=Element->next;
  return 0;
}

//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////


CCellStruct::CCellStruct(void):CSimpleCell()
{
  Filament = new CFilamentStruct*[nfmax];
  for (int i=0;i<nfmax;i++)
    Filament[i]=new CFilamentStruct();
}

CCellStruct::~CCellStruct(void)
{
  for (int i=0;i<nf;i++)
    delete Filament[i];
  delete [] Filament;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
int  CCellStruct::AddFilament(int nactin)
{
  int i;
  int *TMP;

  //check, if nf is in range of F, otherwise - resize F
  if ((nf+1)>nfmax){
    nfmax+=size_step;
    Filament=(CFilamentStruct**)realloc(Filament,sizeof(CFilamentStruct**)*nfmax);
    if (Filament==NULL) return 0;
  }
  Filament[nf] = new CFilamentStruct();

  for (i=0;i<nactin;i++)
    Filament[nf]->AddBarbed(iATF);

  nf++;
  return 1;
}

//---------------------------------------------------------------------------
int  CCellStruct::DelFilament(int ifil)
{
  if (ifil>=nf) return -1;
  nf--;

  delete Filament[ifil];
  Filament[ifil]=Filament[nf];

  return 1;
}

//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int CCellStruct::SimDissociate(int ifil)
{
  if (ifil>=nf)   return -1;

  dis_count++;
  dis_actin+=Filament[ifil]->natf+Filament[ifil]->nadf;

  // actins
  N[iATG]+=Filament[ifil]->natf;
  N[iADG]+=Filament[ifil]->nadf;
  N[iATF]-=Filament[ifil]->natf;
  N[iADF]-=Filament[ifil]->nadf;

  // barbed ends
  switch (Filament[ifil]->b){
   case iATF: N[iFTB]--; break;
   case iADF: N[iFDB]--; break;
   case iCBF: N[iCBF]--; N[iCBG]++; break;
   case iFOF: N[iFOF]--; N[iFOG]++; break;
  }

  // pointed ends
  switch (Filament[ifil]->p){
   case iATF: N[iFTP]--; break;
   case iADF: N[iFDP]--; break;
   case iCPF: N[iCPF]--; N[iCPG]++; break;
  }

  return DelFilament(ifil);
}
//---------------------------------------------------------------------------
// NUCLEATIONS
//---------------------------------------------------------------------------
int CCellStruct::SimSNUC(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;
  N[iATG]-=3;
  N[iATF]+=3;
  N[iFTB]++;
  N[iFTP]++;

  NR[iSNUC]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimFNUC(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(1)) return -2;
  Filament[nf-1]->AddBarbed(iFOF);
  Filament[nf-1]->AddBarbed(iATF);
  Filament[nf-1]->AddBarbed(iATF);


  N[iATG]-=3;

  N[iADF]+=2;
  N[iATF]++;

  N[iFOF]++;
  N[iFOG]--;
  N[iFTP]++;

  NR[iFNUC]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimCBNU(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;
  Filament[nf-1]->AddBarbed(iCBF);

  N[iATG]-=3;
  N[iATF]+=3;
  N[iCBF]++;
  N[iFTP]++;

  NR[iCBNU]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimCPNU(void)
{
  if (N[iATG]<3) return -1;
  // add filament (with check for faults)
  if (!AddFilament(3)) return -2;
  Filament[nf-1]->AddPointed(iCPF);

  N[iATG]-=3;
  N[iATF]+=3;
  N[iFTB]++;
  N[iCPF]++;

  NR[iCPNU]++;
  return 1;
}
//---------------------------------------------------------------------------
// ASSOCIATIONS
//---------------------------------------------------------------------------
int CCellStruct::SimASTB(void)
{
  int tmp,if1;
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))) return -3;

  N[Trans_A_FB(Filament[ifil]->b)]--;
  N[iFTB]++;
  N[iATG]--;
  N[iATF]++;

  Filament[ifil]->AddBarbed(iATF);

  NR[iASTB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimASDB(void)
{
  int tmp,if1;
  int i,j, ifil;
  if (N[iADG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if ((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))) return -3;

  N[Trans_A_FB(Filament[ifil]->b)]--;
  N[iFDB]++;
  N[iADG]--;
  N[iADF]++;

  Filament[ifil]->AddBarbed(iADF);

  NR[iASDB]++;

  return 1;
}

//---------------------------------------------------------------------------
int CCellStruct::SimASTP(void)
{
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFDP]+N[iFTP]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTP]+N[iFDP]);
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if ((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))) return -3;

  N[Trans_A_FP(Filament[ifil]->p)]--;
  N[iFTP]++;
  N[iATG]--;
  N[iATF]++;

  Filament[ifil]->AddPointed(iATF);

  NR[iASTP]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimASDP(void)
{
  int i,j,ifil;
  if (N[iADG]<1) return -1;
  if (N[iFDP]+N[iFTP]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFTP]+N[iFDP]);
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if ((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))) return -3;

  N[Trans_A_FP(Filament[ifil]->p)]--;
  N[iFDP]++;
  N[iADG]--;
  N[iADF]++;

  Filament[ifil]->AddPointed(iADF);

  NR[iASDP]++;

  return 1;
}
//---------------------------------------------------------------------------
// DISSOCIATIONS
//---------------------------------------------------------------------------
int CCellStruct::SimDITB(void)
{
  int i,j, ifil;
  if (N[iATF]<1) return -1;
  if (N[iFTB]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFTB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil]->b==iATF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->b==iATF)) return -3;

  Filament[ifil]->RemBarbed();

  N[iFTB]--;
  N[Trans_A_FB(Filament[ifil]->b)]++;

  N[iATG]++;
  N[iATF]--;

  // if resulted filament is small - dissociate. Otherwise proceed.
  if (Filament[ifil]->nadf+Filament[ifil]->natf<3 && KillShortFilament)
    SimDissociate(ifil);

  NR[iDITB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDIDB(void)
{
  int i,j, ifil;
  if (N[iADF]<1) return -1;
  if (N[iFDB]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil]->b==iADF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->b==iADF)) return -3;

  Filament[ifil]->RemBarbed();
  N[iFDB]--;
  N[Trans_A_FB(Filament[ifil]->b)]++;

  N[iADG]++;
  N[iADF]--;

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil]->nadf+Filament[ifil]->natf<3)&&KillShortFilament)
    SimDissociate(ifil);

  NR[iDIDB]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDITP(void)
{
  int i,j, ifil;
  if (N[iATF]<1) return -1;
  if (N[iFTP]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFTP]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil]->p==iATF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->p==iATF)) return -3;

  Filament[ifil]->RemPointed();
  N[iFTP]--;
  N[Trans_A_FP(Filament[ifil]->p)]++;

  N[iATG]++;
  N[iATF]--;

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil]->nadf+Filament[ifil]->natf<3)&&KillShortFilament)
    SimDissociate(ifil);

   NR[iDITP]++;

  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDIDP(void)
{
  int i,j, ifil;
  if (N[iADF]<1) return -1;
  if (N[iFDP]<1) return -2;
  // select the dissociation filament
  i=RandInt(1,N[iFDP]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    if (Filament[ifil]->p==iADF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->p==iADF)) return -3;

  if (!(Filament[ifil]->RemPointed()==iADF)) return -3;
  N[iFDP]--;
  N[Trans_A_FP(Filament[ifil]->p)]++;

  N[iADG]++;
  N[iADF]--;

  // if resulted filament is small - dissociate. Otherwise proceed.
  if ((Filament[ifil]->nadf+Filament[ifil]->natf<3)&&KillShortFilament)
    SimDissociate(ifil);

  NR[iDIDB]++;

  return 1;
}
//---------------------------------------------------------------------------
// AGING AND ATP-RECHARGE
//---------------------------------------------------------------------------
int CCellStruct::SimTTOD(void)
{
  int i,j,ifil;
  if (N[iATF]<1) return -1;
  // select the aged actin
  i=RandInt(1,N[iATF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++) {
    j+=Filament[ifil]->natf;
    if (j>=i) break;
  }

  // define the actin
  i=RandInt(0,Filament[ifil]->natf-1);
  j=0;
  Filament[ifil]->GoToElement(0);
  while (Filament[ifil]->Element->mol!=iATF)
    Filament[ifil]->Element=Filament[ifil]->Element->next;
  while (j<i){
    if (Filament[ifil]->Element->mol==iATF) j++;
    Filament[ifil]->Element=Filament[ifil]->Element->next;
    while (Filament[ifil]->Element->mol!=iATF)
      Filament[ifil]->Element=Filament[ifil]->Element->next;
  }

  if (Filament[ifil]->Element==NULL){
    i=i/0;
  }

  if (Filament[ifil]->Element->mol!=iATF)
    return -2;

  Filament[ifil]->Element->mol=iADF;
  Filament[ifil]->nadf++;
  Filament[ifil]->natf--;
  N[iATF]--;
  N[iADF]++;

  // track changes in the b-ends due to randomness
  if (Filament[ifil]->b!=Filament[ifil]->Barbed->mol){
    N[Trans_A_FB(Filament[ifil]->b)]--;
    N[Trans_A_FB(Filament[ifil]->Barbed->mol)]++;
    Filament[ifil]->b=Filament[ifil]->Barbed->mol;
  }

  // track changes in the p-ends due to randomness
  if (Filament[ifil]->p!=Filament[ifil]->Pointed->mol){
    N[Trans_A_FP(Filament[ifil]->p)]--;
    N[Trans_A_FP(Filament[ifil]->Pointed->mol)]++;
    Filament[ifil]->p=Filament[ifil]->Pointed->mol;
  }

  NR[iTTOD]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDTOT(void)
{
  if (N[iADG]<1) return -1;
  N[iADG]--;
  N[iATG]++;

  NR[iDTOT]++;
  return 1;
}
//---------------------------------------------------------------------------
// CAPPING
//---------------------------------------------------------------------------
int CCellStruct::SimASCB(void)
{
  int i,j, ifil;
  if (N[iCBG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))) return -3;

  N[Trans_A_FB(Filament[ifil]->b)]--;
  N[iCBG]--;
  N[iCBF]++;

  Filament[ifil]->AddBarbed(iCBF);

  NR[iASCB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDICB(void)
{
  int i,j, ifil;
  if (N[iCBF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iCBF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil]->b==iCBF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->b==iCBF)) return -2;

  Filament[ifil]->RemBarbed();
  N[Trans_A_FB(Filament[ifil]->b)]++;
  N[iCBG]++;
  N[iCBF]--;

  NR[iDICB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimASCP(void)
{
  int i,j, ifil;
  if (N[iCPG]<1) return -1;
  if (N[iFTP]+N[iFDP]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTP]+N[iFDP]); // get random pointed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->p==iATF)||(Filament[ifil]->p==iADF))) return -3;

  N[Trans_A_FP(Filament[ifil]->p)]--;
  N[iCPG]--;
  N[iCPF]++;
  Filament[ifil]->AddPointed(iCPF);

  NR[iASCP]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDICP(void)
{
  int i,j, ifil;
  if (N[iCPF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iCPF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil]->p==iCPF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->p==iCPF)) return -2;

  Filament[ifil]->RemPointed();
  N[Trans_A_FP(Filament[ifil]->p)]++;
  N[iCPG]++;
  N[iCPF]--;

  NR[iDICP]++;
  return 1;
}
//---------------------------------------------------------------------------
// FORMIN OPERATION
//---------------------------------------------------------------------------
int CCellStruct::SimASFB(void)
{
  int i,j, ifil;
  if (N[iFOG]<1) return -1;
  if (N[iFTB]+N[iFDB]<1) return -2;
  // select the filament to be capped
  i=RandInt(1,N[iFTB]+N[iFDB]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if ((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))
      j++;
    if (j>=i) break;
  }
  if (!((Filament[ifil]->b==iATF)||(Filament[ifil]->b==iADF))) return -3;

  N[Trans_A_FB(Filament[ifil]->b)]--;
  Filament[ifil]->AddBarbed(iFOF);

  N[iFOG]--;
  N[iFOF]++;

  NR[iASFB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimDIFB(void)
{
  int i,j, ifil;
  if (N[iFOF]<1) return -1;
  // select the filament to be uncapped
  i=RandInt(1,N[iFOF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil]->b==iFOF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->b==iFOF)) return -2;

  Filament[ifil]->RemBarbed();
  N[Trans_A_FB(Filament[ifil]->b)]++;

  N[iFOG]++;
  N[iFOF]--;

  NR[iDIFB]++;
  return 1;
}
//---------------------------------------------------------------------------
int CCellStruct::SimFASB(void)
{
  int i,j, ifil;
  if (N[iATG]<1) return -1;
  if (N[iFOF]<1) return -2;
  // select the growing filament
  i=RandInt(1,N[iFOF]); // get random barbed end
  j=0;
  // define the filament
  for (ifil=0;ifil<nf;ifil++){
    if (Filament[ifil]->b==iFOF)
      j++;
    if (j>=i) break;
  }
  if (!(Filament[ifil]->b==iFOF)) return -3;

  Filament[ifil]->AddBarbed(iADF);
  N[iATG]--;
  N[iADF]++;

  NR[iFASB]++;
  return 1;
}
//----------------------------------------------------------------------------
int CCellStruct::GetLength3(void)
{
  int ifil,n3=0;
  for (ifil=0;ifil<nf;ifil++)
   if (Filament[ifil]->nadf+Filament[ifil]->natf == 3)
     n3++;
  return n3;
}
//----------------------------------------------------------------------------
int CCellStruct::RetFilLen(int nm){
	return Filament[nm]->nadf+Filament[nm]->natf;
}
//----------------------------------------------------------------------------
int CCellStruct::GetMinLength(void)
{
  int minl=1e6;
  for (int ifil=0; ifil<nf; ifil++){
    if (Filament[ifil]->nadf+Filament[ifil]->natf < minl)
      minl=Filament[ifil]->nadf+Filament[ifil]->natf;
  }
  return minl;
}
//----------------------------------------------------------------------------
int CCellStruct::GetMaxLength(void)
{
  int maxl=0;
  for (int ifil=0; ifil<nf; ifil++){
    if (Filament[ifil]->nadf+Filament[ifil]->natf > maxl)
      maxl=Filament[ifil]->nadf+Filament[ifil]->natf;
  }
  return maxl;
}

//----------------------------------------------------------------------------
void CCellStruct::GetLengthDistribution(int *B, int *F, int _vmin, int _vmax, int nbins)
{
  int vmin, vmax;
  int s;
  int i,ifil;

  if (_vmin<0) vmin=GetMinLength();
  else vmin=_vmin;
  if (_vmax<0) vmax=GetMaxLength();
  else vmax=_vmax;

  for (i=0;i<nbins;i++) {
    B[i]=vmin+int((double)(vmax-vmin)*(i+1.0)/nbins);
    F[i]=0;
  }

  for (ifil=0; ifil<nf; ifil++){
    for (i=0;i<nbins;i++)
      if (Filament[ifil]->nadf+Filament[ifil]->natf<=B[i]){
        F[i]=F[i]+1;
        break;
      }
  }

}

//---------------------------------------------------------------------------


