//classes CFilamentStruct, CCellStruct
//---------------------------------------------------------------------------
#ifndef CellStructH
#define CellStructH
//---------------------------------------------------------------------------
#include "SimpleCell.h"

char Reform_GF_FG (char mol); // change index: from F/G molecules to G/F
char Trans_A_FB (char mol);  // change index: given actin return end-index
char Trans_A_FP (char mol);  // change index: given actin return end-index


//extern struct TElement;
struct TElement {
  char mol;        // type of molecule
  TElement *prev;  // pointer to previous
  TElement *next;  // pointer to next
};

class CFilamentStruct{
 public:
  TElement * Barbed;   // stack head
  TElement * Pointed;  // stack tail
  TElement * Element; // temporary element

  int n;    // number of molecules in the stack
  int natf; // number of ATP-actines
  int nadf; // number of ADP-actines
  char b;   // barbed
  char p;   // pointed
  CFilamentStruct(void);
  ~CFilamentStruct(void);

  // Add and Remove.
  void AddBarbed(char mol);
  void AddPointed(char mol);
  char RemBarbed(void);
  char RemPointed(void);

  int GoToElement(int cnt);
};

class CCellStruct : public CSimpleCell {
 public:

  CFilamentStruct **Filament;


  int itmin;    // index of the minimal time

  CCellStruct(void);
  virtual ~CCellStruct(void);

  // calculate mean filament length
  virtual int GetMinLength(void);
  virtual int GetMaxLength(void);
  virtual void GetLengthDistribution(int *B, int *F, int vmin, int vmax, int nbins);

  // Return number of filaments with l=3
  virtual int GetLength3(void);

  //Return length of filament with index nm
  virtual int RetFilLen(int nm);

  // add filament to the system
  virtual int AddFilament(int nactin);
  // delete filament from the system
  virtual int DelFilament(int ifil);


  // Simulate dissociation of the filament #ifil
  virtual int SimDissociate(int ifil);

  // Simulate stochastic filament formation
  virtual int SimSNUC(void);
  // Simulate formin filament formation
  virtual int SimFNUC(void);
  // Simulate barbed capping protein nuctleation
  virtual int SimCBNU(void);
  // Simulate pointed capping protein nuctleation
  virtual int SimCPNU(void);


  // Simulate ATP-actin association at barbed end
  virtual int SimASTB(void);
  // Simulate ADP-actin association at barbed end
  virtual int SimASDB(void);
  // Simulate ATP-actin association at pointed end
  virtual int SimASTP(void);
  // Simulate ADP-actin association at pointed end
  virtual int SimASDP(void);


  // Simulate dissociation at ATP-barbed end
  virtual int SimDITB(void);
  // Simulate dissociation at ADP-barbed end
  virtual int SimDIDB(void);
  // Simulate dissociation at ATP-pointed end
  virtual int SimDITP(void);
  // Simulate dissociation at ADP-pointed end
  virtual int SimDIDP(void);

  // Simulate aging of F-actins
  virtual int SimTTOD(void);
  // Simulate recharge of G-actins
  virtual int SimDTOT(void);

  // Capping of barbed end
  virtual int SimASCB(void);
  // Uncapping of barbed end
  virtual int SimDICB(void);
  // Capping of pointed end
  virtual int SimASCP(void);
  // Uncapping of pointed end
  virtual int SimDICP(void);

  // Association of formin to barbed end
  virtual int SimASFB(void);
  // Dissociation of formin from barbed end
  virtual int SimDIFB(void);
  // Formin-initiated barbed-end association
  virtual int SimFASB(void);


};

#endif




