//---------------------------------------------------------------------------
#include <stddef.h>
#include <math.h>
#include <mem.h>

#pragma hdrstop


#include <LibRand.h>
#include <LibBiophysics.h>

#include "SimpleCell.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
//---------------------------------------------------------------------------
CSimpleCell::CSimpleCell(void)
{
  v=t=0;
  nf=0;
  nfmax=size_step;
  KillShortFilament=true;

  memset(N, 0,nMTypes*sizeof(N[0]));
  memset(K, 0,nRTypes*sizeof(K[0]));
  memset(NR,0,nRTypes*sizeof(NR[0]));

  dis_count=dis_actin=0;
  dis_count0=dis_actin0=0;
  //for Gibson-Bruck method
  pA=A;
  pA_old=A_old;

}
//---------------------------------------------------------------------------
CSimpleCell::~CSimpleCell(void){}
//---------------------------------------------------------------------------
int  CSimpleCell::SimulateFR(double tmax)
{
  int state;

  // some constants (to speed up)
  double v1uN_AV1=v*uN_AV;
  double v2uN_AV2=v*v*uN_AV*uN_AV;
  double v3uN_AV3=v*v*v*uN_AV*uN_AV*uN_AV;

  //minimal time
  double tmin;

  int i;    // arbitrary index
  int ireact=0; // reaction number

  int ifil;

  // main loop
  while (t<=tmax) {
    ireact++;

     A[iSNUC]=K[iSNUC]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v2uN_AV2;
     A[iFNUC]=K[iFNUC]*N[iFOG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     A[iCBNU]=K[iCBNU]*N[iCBG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     A[iCPNU]=K[iCPNU]*N[iCPG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;

     A[iASTB]=K[iASTB]*(N[iFTB]+N[iFDB])*N[iATG]/v1uN_AV1;
     A[iASDB]=K[iASDB]*(N[iFTB]+N[iFDB])*N[iADG]/v1uN_AV1;
     A[iASTP]=K[iASTP]*(N[iFTP]+N[iFDP])*N[iATG]/v1uN_AV1;
     A[iASDP]=K[iASDP]*(N[iFTP]+N[iFDP])*N[iADG]/v1uN_AV1;

     A[iDITB]=K[iDITB]*N[iFTB];
     A[iDIDB]=K[iDIDB]*N[iFDB];
     A[iDITP]=K[iDITP]*N[iFTP];
     A[iDIDP]=K[iDIDP]*N[iFDP];

     A[iTTOD]=K[iTTOD]*N[iATF];
     A[iDTOT]=K[iDTOT]*N[iADG];

     A[iASCB]=K[iASCB]*(N[iFTB]+N[iFDB])*N[iCBG]/v1uN_AV1;
     A[iASCP]=K[iASCP]*(N[iFTP]+N[iFDP])*N[iCPG]/v1uN_AV1;
     A[iDICB]=K[iDICB]*N[iCBF];
     A[iDICP]=K[iDICP]*N[iCPF];

     A[iASFB]=K[iASFB]*(N[iFTB]+N[iFDB])*N[iFOG]/v1uN_AV1;
     A[iDIFB]=K[iDIFB]*N[iFOF];
     A[iFASB]=K[iFASB]*N[iATG]*N[iFOF]/v1uN_AV1;



    //check for zeros
    for (i=0;i<nRTypes;i++) if (A[i]<epsilon) A[i]=epsilon;

    //generate reaction times
    for (i=0;i<nRTypes;i++) T[i]=-log(randu())/A[i];

    //select minimal time
    tmin=1e10;
    for (i=0;i<nRTypes;i++)
      if (T[i]<tmin) {
        tmin=T[i];
        itmin=i;
      };
    t+=tmin;

    //perform the reaction with minimal time
    switch (itmin) {
      case iSNUC: state=SimSNUC(); break;
      case iFNUC: state=SimFNUC(); break;
      case iCBNU: state=SimCBNU(); break;
      case iCPNU: state=SimCPNU(); break;

      case iASTB: state=SimASTB(); break;
      case iASDB: state=SimASDB(); break;
      case iASTP: state=SimASTP(); break;
      case iASDP: state=SimASDP(); break;

      case iDITB: state=SimDITB(); break;
      case iDIDB: state=SimDIDB(); break;
      case iDITP: state=SimDITP(); break;
      case iDIDP: state=SimDIDP(); break;

      case iTTOD: state=SimTTOD(); break;
      case iDTOT: state=SimDTOT(); break;

      case iASCB: state=SimASCB(); break;
      case iASCP: state=SimASCP(); break;
      case iDICB: state=SimDICB(); break;
      case iDICP: state=SimDICP(); break;

      case iASFB: state=SimASFB(); break;
      case iDIFB: state=SimDIFB(); break;
      case iFASB: state=SimFASB(); break;

    }//{switch (itmin)}


    // check for errors during reaction
    if (state<=0) {
      // !!! ERROR !!!
      itmin=itmin;

      state/=state*0.0;
    }

  }//{while ((t<tmax)&&(ireact<nreact))}
  return state;
}
//---------------------------------------------------------------------------
int  CSimpleCell::SimulateDG(double tmax)
{
  int state;

  // some constants (to speed up)
  double v1uN_AV1=v*uN_AV;
  double v2uN_AV2=v*v*uN_AV*uN_AV;
  double v3uN_AV3=v*v*v*uN_AV*uN_AV*uN_AV;

  //random number
  double rn;


  //total speed
  double a_sum;

  //work variable
  double a_r;

  int i;    // arbitrary index
  int ireact=0; // reaction number

  int ifil;

  // main loop
  while (t<=tmax) {
    ireact++;

     A[iSNUC]=K[iSNUC]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v2uN_AV2;
     A[iFNUC]=K[iFNUC]*N[iFOG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     A[iCBNU]=K[iCBNU]*N[iCBG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     A[iCPNU]=K[iCPNU]*N[iCPG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;

     A[iASTB]=K[iASTB]*(N[iFTB]+N[iFDB])*N[iATG]/v1uN_AV1;
     A[iASDB]=K[iASDB]*(N[iFTB]+N[iFDB])*N[iADG]/v1uN_AV1;
     A[iASTP]=K[iASTP]*(N[iFTP]+N[iFDP])*N[iATG]/v1uN_AV1;
     A[iASDP]=K[iASDP]*(N[iFTP]+N[iFDP])*N[iADG]/v1uN_AV1;

     A[iDITB]=K[iDITB]*N[iFTB];
     A[iDIDB]=K[iDIDB]*N[iFDB];
     A[iDITP]=K[iDITP]*N[iFTP];
     A[iDIDP]=K[iDIDP]*N[iFDP];

     A[iTTOD]=K[iTTOD]*N[iATF];
     A[iDTOT]=K[iDTOT]*N[iADG];

     A[iASCB]=K[iASCB]*(N[iFTB]+N[iFDB])*N[iCBG]/v1uN_AV1;
     A[iASCP]=K[iASCP]*(N[iFTP]+N[iFDP])*N[iCPG]/v1uN_AV1;
     A[iDICB]=K[iDICB]*N[iCBF];
     A[iDICP]=K[iDICP]*N[iCPF];

     A[iASFB]=K[iASFB]*(N[iFTB]+N[iFDB])*N[iFOG]/v1uN_AV1;
     A[iDIFB]=K[iDIFB]*N[iFOF];
     A[iFASB]=K[iFASB]*N[iATG]*N[iFOF]/v1uN_AV1;

    //check for zeros
    a_sum=0;
    for (i=0;i<nRTypes;i++){
      if (A[i]<epsilon) A[i]=epsilon;
      a_sum+=A[i];
    }

    //select reaction number
    rn=randu()*a_sum;
    a_r=0;
    for (i=0;i<nRTypes;i++){
      a_r+=A[i];
      if (a_r>rn) {
        itmin=i;
        break;
      };
    }

    //determine reaction time
    t+=-log(randu())/a_sum;

    //perform the reaction with minimal time
    switch (itmin) {
      case iSNUC: state=SimSNUC(); break;
      case iFNUC: state=SimFNUC(); break;
      case iCBNU: state=SimCBNU(); break;
      case iCPNU: state=SimCPNU(); break;

      case iASTB: state=SimASTB(); break;
      case iASDB: state=SimASDB(); break;
      case iASTP: state=SimASTP(); break;
      case iASDP: state=SimASDP(); break;

      case iDITB: state=SimDITB(); break;
      case iDIDB: state=SimDIDB(); break;
      case iDITP: state=SimDITP(); break;
      case iDIDP: state=SimDIDP(); break;

      case iTTOD: state=SimTTOD(); break;
      case iDTOT: state=SimDTOT(); break;

      case iASCB: state=SimASCB(); break;
      case iASCP: state=SimASCP(); break;
      case iDICB: state=SimDICB(); break;
      case iDICP: state=SimDICP(); break;

      case iASFB: state=SimASFB(); break;
      case iDIFB: state=SimDIFB(); break;
      case iFASB: state=SimFASB(); break;

    }//{switch (itmin)}


    // check for errors during reaction
    if (state<=0) {
      // !!! ERROR !!!
      itmin=itmin;
      state/=state*0.0;
    }

  }//{while ((t<tmax)&&(ireact<nreact))}
  return state;
}

//---------------------------------------------------------------------------
int  CSimpleCell::SimulateGB(double tmax)
{
  int state;

  // some constants (to speed up)
  double v1uN_AV1=v*uN_AV;
  double v2uN_AV2=v*v*uN_AV*uN_AV;
  double v3uN_AV3=v*v*v*uN_AV*uN_AV*uN_AV;


  double *p;



  //minimal time
  double tmin;

  int i;    // arbitrary index
  int ireact=0; // reaction number

  int ifil;

  memset(t1,0,nRTypes*sizeof(t1[0]));

  //first iteration of Gibson-Bruck method
  if (t<=epsilon2){//t<=tmax
    ireact++;

    pA[iSNUC]=K[iSNUC]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v2uN_AV2;
    pA[iFNUC]=K[iFNUC]*N[iFOG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
    pA[iCBNU]=K[iCBNU]*N[iCBG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
    pA[iCPNU]=K[iCPNU]*N[iCPG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;

    pA[iASTB]=K[iASTB]*(N[iFTB]+N[iFDB])*N[iATG]/v1uN_AV1;
    pA[iASDB]=K[iASDB]*(N[iFTB]+N[iFDB])*N[iADG]/v1uN_AV1;
    pA[iASTP]=K[iASTP]*(N[iFTP]+N[iFDP])*N[iATG]/v1uN_AV1;
    pA[iASDP]=K[iASDP]*(N[iFTP]+N[iFDP])*N[iADG]/v1uN_AV1;

    pA[iDITB]=K[iDITB]*N[iFTB];
    pA[iDIDB]=K[iDIDB]*N[iFDB];
    pA[iDITP]=K[iDITP]*N[iFTP];
    pA[iDIDP]=K[iDIDP]*N[iFDP];

    pA[iTTOD]=K[iTTOD]*N[iATF];
    pA[iDTOT]=K[iDTOT]*N[iADG];

    pA[iASCB]=K[iASCB]*(N[iFTB]+N[iFDB])*N[iCBG]/v1uN_AV1;
    pA[iASCP]=K[iASCP]*(N[iFTP]+N[iFDP])*N[iCPG]/v1uN_AV1;
    pA[iDICB]=K[iDICB]*N[iCBF];
    pA[iDICP]=K[iDICP]*N[iCPF];

    pA[iASFB]=K[iASFB]*(N[iFTB]+N[iFDB])*N[iFOG]/v1uN_AV1;
    pA[iDIFB]=K[iDIFB]*N[iFOF];
    pA[iFASB]=K[iFASB]*N[iATG]*N[iFOF]/v1uN_AV1;

    //check for zeros
    for (i=0;i<nRTypes;i++)
      if (pA[i]<epsilon){
        pA[i]=epsilon;
        if (t1[i]<epsilon2)
          t1[i]=t;
      }

    //generate reaction times
    for (i=0;i<nRTypes;i++){
      T[i]=t-log(randu())/pA[i];
    }

    //select minimal time
    tmin=1e10;
    for (i=0;i<nRTypes;i++)
      if (T[i]<tmin) {
        tmin=T[i];
        itmin=i;
      };
    t=tmin;

    //perform the reaction with minimal time
    switch (itmin) {
      case iSNUC: state=SimSNUC(); break;
      case iFNUC: state=SimFNUC(); break;
      case iCBNU: state=SimCBNU(); break;
      case iCPNU: state=SimCPNU(); break;

      case iASTB: state=SimASTB(); break;
      case iASDB: state=SimASDB(); break;
      case iASTP: state=SimASTP(); break;
      case iASDP: state=SimASDP(); break;

      case iDITB: state=SimDITB(); break;
      case iDIDB: state=SimDIDB(); break;
      case iDITP: state=SimDITP(); break;
      case iDIDP: state=SimDIDP(); break;

      case iTTOD: state=SimTTOD(); break;
      case iDTOT: state=SimDTOT(); break;

      case iASCB: state=SimASCB(); break;
      case iASCP: state=SimASCP(); break;
      case iDICB: state=SimDICB(); break;
      case iDICP: state=SimDICP(); break;

      case iASFB: state=SimASFB(); break;
      case iDIFB: state=SimDIFB(); break;
      case iFASB: state=SimFASB(); break;

    }//{switch (itmin)}


    // check for errors during reaction
    if (state<=0) {
      // !!! ERROR !!!
      itmin=itmin;
      state/=state*0.0;
    }
  }//if(t<=tmax)

  // main loop
  while (t<=tmax) {
    ireact++;

     p=pA;
     pA=pA_old;
     pA_old=p;

     pA[iSNUC]=K[iSNUC]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v2uN_AV2;
     pA[iFNUC]=K[iFNUC]*N[iFOG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     pA[iCBNU]=K[iCBNU]*N[iCBG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;
     pA[iCPNU]=K[iCPNU]*N[iCPG]*N[iATG]*(N[iATG]-1)*(N[iATG]-2)/v3uN_AV3;

     pA[iASTB]=K[iASTB]*(N[iFTB]+N[iFDB])*N[iATG]/v1uN_AV1;
     pA[iASDB]=K[iASDB]*(N[iFTB]+N[iFDB])*N[iADG]/v1uN_AV1;
     pA[iASTP]=K[iASTP]*(N[iFTP]+N[iFDP])*N[iATG]/v1uN_AV1;
     pA[iASDP]=K[iASDP]*(N[iFTP]+N[iFDP])*N[iADG]/v1uN_AV1;

     pA[iDITB]=K[iDITB]*N[iFTB];
     pA[iDIDB]=K[iDIDB]*N[iFDB];
     pA[iDITP]=K[iDITP]*N[iFTP];
     pA[iDIDP]=K[iDIDP]*N[iFDP];

     pA[iTTOD]=K[iTTOD]*N[iATF];
     pA[iDTOT]=K[iDTOT]*N[iADG];

     pA[iASCB]=K[iASCB]*(N[iFTB]+N[iFDB])*N[iCBG]/v1uN_AV1;
     pA[iASCP]=K[iASCP]*(N[iFTP]+N[iFDP])*N[iCPG]/v1uN_AV1;
     pA[iDICB]=K[iDICB]*N[iCBF];
     pA[iDICP]=K[iDICP]*N[iCPF];

     pA[iASFB]=K[iASFB]*(N[iFTB]+N[iFDB])*N[iFOG]/v1uN_AV1;
     pA[iDIFB]=K[iDIFB]*N[iFOF];
     pA[iFASB]=K[iFASB]*N[iATG]*N[iFOF]/v1uN_AV1;

    //check for zeros
    for (i=0;i<nRTypes;i++)
      if (pA[i]<(2*epsilon)){
        pA[i]=epsilon;
        if (t1[i]<epsilon2)
          t1[i]=t;
      }

    T[itmin]=t-log(randu())/pA[itmin];

    for (i=0;i<nRTypes;i++)
      if (i!=itmin)
        if (pA[i]>epsilon2)
          if (pA_old[i]<epsilon2)
            T[i]=pA_old[i]/pA[i]*(T[i]-t)+t;
          else{
            T[i]=t-log(randu())/pA[i];
          }
        else{
          //if (pA_old[i]<(epsilon2))
          T[i]=infin;
          //pA[i]=pA_old[i];
        }



    //select minimal time
    tmin=1e10;
    for (i=0;i<nRTypes;i++)
      if (T[i]<tmin) {
        tmin=T[i];
        itmin=i;
      };
    t=tmin;

    //perform the reaction with minimal time
    switch (itmin) {
      case iSNUC: state=SimSNUC(); break;
      case iFNUC: state=SimFNUC(); break;
      case iCBNU: state=SimCBNU(); break;
      case iCPNU: state=SimCPNU(); break;

      case iASTB: state=SimASTB(); break;
      case iASDB: state=SimASDB(); break;
      case iASTP: state=SimASTP(); break;
      case iASDP: state=SimASDP(); break;

      case iDITB: state=SimDITB(); break;
      case iDIDB: state=SimDIDB(); break;
      case iDITP: state=SimDITP(); break;
      case iDIDP: state=SimDIDP(); break;

      case iTTOD: state=SimTTOD(); break;
      case iDTOT: state=SimDTOT(); break;

      case iASCB: state=SimASCB(); break;
      case iASCP: state=SimASCP(); break;
      case iDICB: state=SimDICB(); break;
      case iDICP: state=SimDICP(); break;

      case iASFB: state=SimASFB(); break;
      case iDIFB: state=SimDIFB(); break;
      case iFASB: state=SimFASB(); break;

    }//{switch (itmin)}


    // check for errors during reaction
    if (state<=0) {
      // !!! ERROR !!!
      itmin=itmin;
      state/=state*0.0;
    }

  }//{while ((t<tmax)&&(ireact<nreact))}
  return state;
}
//////////////////////////////////////////////////////////////////////////////

CData::CData(int _n)
{
  n=_n;

  T=new double[n];
  TIME=new double[n];

  D=new double[n];
  DATA=new double[n];
  min=max=timeshift=background=0;
  amplitude=1;
}
//----------------------------------------------------------------------------
CData::~CData()
{
  delete [] T;
  delete [] TIME;
  delete [] D;
  delete [] DATA;
}
//----------------------------------------------------------------------------
void CData::SetParam(double _tsh,double _bgr, double _amp)
{
  timeshift=_tsh;
  background=_bgr;
  amplitude=_amp;
  GetMin();
  GetMax();
}

//----------------------------------------------------------------------------
double CData::GetMin(void)
{
  min=1e20;
  for (int i=0;i<n;i++)
    if (DATA[i]<min) min=DATA[i];
  return min;
}
//----------------------------------------------------------------------------
double CData::GetMax(void)
{
  max=-1e20;
  for (int i=0;i<n;i++)
    if (DATA[i]>max) max=DATA[i];
  return max;
}
//----------------------------------------------------------------------------
void CData::Rescale(bool _CalcMinMax)
{
  if (_CalcMinMax) {
    GetMax();
    GetMin();
  }
  double a;
  if (fabs(max-background)>epsilon) a=max-min;
                               else a=1;
  a=amplitude/a;
  for (int i=0;i<n;i++){
    D[i]=(DATA[i]-min)*a+background;
    T[i]=TIME[i]+timeshift;
  }
}


//----------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////////

CResults::CResults(int _nd, int _nt)
{
  nt=_nt;
  nd=_nd;

  int id;

  T=new double[nt];

  DATA=new double*[nd];
  for (id=0;id<nd;id++)
    DATA[id]=new double[nt];

}
//----------------------------------------------------------------------------
CResults::~CResults()
{
  int id;
  delete [] T;
  for (id=0;id<nd;id++)
    delete [] DATA[id];
  delete [] DATA;
}
//----------------------------------------------------------------------------



